==> akshare-0.4.4.tar.gz: akshare-0.4.4/akshare/stock/zh_stock_kcb_sina.py <==
# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Author: Albert King
date: 2019/10/30 11:28
contact: jindaxiang@163.com
desc: 新浪财经-科创板-实时行情数据和历史行情数据(包含前复权和后复权因子)
优化: 在科创板行情的获取上采用多线程模式(新浪会封IP, 不再优化)
"""
import datetime
import re

import demjson
import pandas as pd
import requests

from akshare.stock.cons import (zh_sina_kcb_stock_payload,
                                zh_sina_kcb_stock_url,
                                zh_sina_kcb_stock_count_url,
                                zh_sina_kcb_stock_hist_url,
                                zh_sina_kcb_stock_hfq_url,
                                zh_sina_kcb_stock_qfq_url)


def get_zh_kcb_page_count():
    """
    所有股票的总页数
    http://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: int 需要抓取的股票总页数
    """
    res = requests.get(zh_sina_kcb_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_zh_kcb_spot():
    """
    从新浪财经-A股获取所有A股的实时行情数据, 大量抓取容易封IP
    http://vip.stock.finance.sina.com.cn/mkt/#qbgg_hk
    :return: pandas.DataFrame
              symbol    code  name    trade pricechange changepercent      buy  \
    0   sh688001  688001  华兴源创   36.020      -1.550        -4.126   36.020
    1   sh688002  688002  睿创微纳   34.560      -0.590        -1.679   34.550
    2   sh688003  688003  天准科技   26.800      -0.400        -1.471   26.740
    3   sh688005  688005  容百科技   30.930       0.000         0.000    0.000
    4   sh688006  688006  杭可科技   39.400      -1.610        -3.926   39.400
    5   sh688007  688007  光峰科技   27.780      -0.400        -1.419   27.780
    6   sh688008  688008  澜起科技   61.130      -1.030        -1.657   61.120
    7   sh688009  688009  中国通号    7.550      -0.040        -0.527    7.540
    8   sh688010  688010  福光股份   45.120       0.260         0.580   45.120
    9   sh688011  688011  新光光电   40.620      -0.550        -1.336   40.620
    10  sh688012  688012  中微公司   66.760      -2.710        -3.901   66.760
    11  sh688015  688015  交控科技   34.540      -0.120        -0.346   34.540
    12  sh688016  688016  心脉医疗  138.800       2.520         1.849  138.580
    13  sh688018  688018  乐鑫科技  140.160      -0.840        -0.596  140.160
    14  sh688019  688019  安集科技  113.800      -1.560        -1.352  113.800
    15  sh688020  688020  方邦股份   78.650       0.250         0.319   78.640
    16  sh688021  688021   N奥福   33.210       7.040        26.901   33.200
    17  sh688022  688022  瀚川智能   41.040      -0.710        -1.701   41.040
    18  sh688023  688023  安恒信息   88.510       9.450        11.953   88.510
    19  sh688025  688025   杰普特   45.140      -1.230        -2.653   45.010
    20  sh688028  688028   沃尔德   60.010      -0.100        -0.166   60.010
    21  sh688029  688029  南微医学  180.120       2.820         1.591  180.120
    22  sh688030  688030  山石网科   40.500       0.610         1.529   40.500
    23  sh688033  688033  天宜上佳   28.880      -0.130        -0.448   28.810
    24  sh688036  688036  传音控股   40.910      -0.550        -1.327   40.910
    25  sh688058  688058   宝兰德   87.700      -2.300        -2.556   87.670
    26  sh688066  688066  航天宏图   37.920       0.020         0.053   37.920
    27  sh688068  688068  热景生物   47.060      -0.940        -1.958   47.050
    28  sh688088  688088  虹软科技   42.170      -0.150        -0.354   42.170
    29  sh688098  688098  申联生物   17.590      -1.010        -5.430   17.580
    30  sh688099  688099  晶晨股份   53.800       0.060         0.112   53.800
    31  sh688108  688108  赛诺医疗   17.410      -1.080        -5.841   17.410
    32  sh688116  688116  天奈科技   28.320      -0.210        -0.736   28.310
    33  sh688122  688122  西部超导   30.910       0.400         1.311   30.900
    34  sh688128  688128  中国电研   21.930      -4.990       -18.536   21.920
    35  sh688139  688139  海尔生物   26.670      -0.930        -3.370   26.660
    36  sh688168  688168   安博通   89.690       0.490         0.549   89.560
    37  sh688188  688188  柏楚电子  132.020       2.250         1.734  132.020
    38  sh688199  688199  久日新材   66.350      -4.610        -6.497   66.350
    39  sh688202  688202   美迪西   66.380      -4.620        -6.507   66.250
    40  sh688288  688288   N鸿泉   31.260       6.270        25.090   31.260
    41  sh688299  688299   N长阳   17.860       4.150        30.270   17.860
    42  sh688321  688321  微芯生物   54.110       2.270         4.379   54.110
    43  sh688333  688333   铂力特   55.090      -1.770        -3.113   55.090
    44  sh688363  688363   N华熙   85.100      37.310        78.071   85.100
    45  sh688366  688366  昊海生科   89.850      -0.010        -0.011   89.830
    46  sh688368  688368  晶丰明源   64.300      -0.980        -1.501   64.210
    47  sh688369  688369  致远互联   64.000      -0.610        -0.944   64.000
    48  sh688388  688388  嘉元科技   44.740       1.040         2.380   44.710
    49  sh688389  688389  普门科技   18.100      -1.530        -7.794   18.090
           sell settlement     open     high      low    volume      amount  \
    0    36.030     37.570   37.850   37.920   35.810   2380984    87476401
    1    34.560     35.150   35.170   35.690   34.560   2103123    73990175
    2    26.800     27.200   27.200   27.360   26.600   2057578    55542380
    3     0.000     30.930    0.000    0.000    0.000         0           0
    4    39.410     41.010   41.160   42.000   38.810   4930690   198288639
    5    27.790     28.180   28.180   28.480   27.540   1713538    48026144
    6    61.130     62.160   62.320   62.620   60.900   2331354   143703073
    7     7.550      7.590    7.570    7.660    7.530  20843563   157970856
    8    45.140     44.860   44.710   46.200   44.550   2088806    94878090
    9    40.630     41.170   41.030   41.730   40.300    924537    37802171
    10   66.770     69.470   69.050   70.700   66.360   3166810   217462568
    11   34.650     34.660   34.980   35.530   34.410   1690038    59026743
    12  138.800    136.280  136.030  141.680  135.010    890307   123543447
    13  140.190    141.000  140.000  145.000  138.300    949958   134262776
    14  113.810    115.360  114.000  116.850  113.520    543447    62396461
    15   78.650     78.400   77.540   80.270   77.530    880072    69705273
    16   33.210     26.170   30.000   34.900   30.000  13693192   444011825
    17   41.060     41.750   41.750   42.480   40.900   1143725    47732979
    18   88.530     79.060   77.400   91.900   77.400   7154366   605779685
    19   45.140     46.370   45.790   46.850   44.950   2533248   115958362
    20   60.050     60.110   60.400   61.360   59.200   1243836    74998220
    21  180.150    177.300  178.100  181.010  175.720   1649325   294724332
    22   40.510     39.890   40.000   41.960   39.700   5295281   215776050
    23   28.880     29.010   29.210   29.440   28.580   1118531    32345154
    24   40.920     41.460   41.100   41.990   40.690   4809262   198377404
    25   87.700     90.000   88.150   89.800   86.550   1759660   154998681
    26   37.930     37.900   37.910   38.830   37.850   1361196    52129856
    27   47.060     48.000   47.960   48.330   46.500   1113358    52776362
    28   42.180     42.320   42.420   42.690   41.900   1706984    72205682
    29   17.590     18.600   18.040   18.590   17.460   8656313   155253889
    30   53.820     53.740   53.610   55.430   53.550   1811039    98609660
    31   17.420     18.490   17.500   18.450   17.220   9959654   175741647
    32   28.320     28.530   28.600   29.250   28.200   4119339   118200733
    33   30.910     30.510   30.580   31.550   30.580   1790005    55652309
    34   21.930     26.920   23.800   24.200   21.880  21824232   505020080
    35   26.670     27.600   27.150   27.990   26.660   9026072   245111328
    36   89.690     89.200   89.190   92.730   88.880    790196    71559830
    37  132.040    129.770  129.490  135.050  128.600   1745272   232065547
    38   66.360     70.960   66.950   68.230   66.010   8803968   588840957
    39   66.380     71.000   68.200   71.000   64.220   5959905   404831463
    40   31.270     24.990   35.000   35.000   30.500  15451343   500316962
    41   17.870     13.710   18.450   19.680   17.220  47843863   862422893
    42   54.120     51.840   51.900   55.740   51.900   4021213   217774905
    43   55.100     56.860   56.100   57.800   55.070   1591126    89698024
    44   85.120     47.790   78.000   92.200   78.000  33823982  2807767312
    45   89.850     89.860   88.530   92.740   88.530   2802240   255113439
    46   64.300     65.280   65.000   65.860   64.130   1157674    75158797
    47   64.010     64.610   64.100   66.330   63.500   2663678   172052487
    48   44.740     43.700   43.360   45.960   42.910   4445248   199015034
    49   18.100     19.630   18.500   19.090   17.980  15342408   282404361
        ticktime      per      pb        mktcap            nmc  turnoverratio
    0   15:29:59   53.761   7.594  1.444402e+06  130570.987160        6.56831
    1   15:29:59   92.779   6.862  1.537920e+06  178220.760192        4.07831
    2   15:29:59   40.606   3.309  5.188480e+05  118398.334440        4.65742
    3   15:29:59   55.232   3.128  1.371083e+06  125999.748231        0.00000
    4   15:29:59   49.560   7.124  1.579940e+06  144936.729680       13.40372
    5   15:29:59   38.055   6.575  1.254418e+06  158881.912314        2.99607
    6   15:29:59   70.264   9.642  6.906552e+06  454714.502468        3.13418
    7   15:29:59   19.868   2.101  7.995313e+06  894199.803000        1.75989
    8   15:29:59   56.669   3.976  6.929617e+05  159931.061568        5.89297
    9   15:29:59   41.876   3.449  4.062000e+05   92511.810342        4.05945
    10  15:29:59  333.800   9.672  3.570740e+06  323295.801348        6.53941
    11  15:29:59   62.800   5.500  5.526400e+05  113077.684082        5.16228
    12  15:29:59   82.619   9.637  9.990567e+05  194054.225760        6.36805
    13  15:29:59   89.576   7.245  1.121280e+06  244398.589824        5.44791
    14  15:29:59  100.708   6.963  6.043734e+05  131722.999280        4.69502
    15  15:29:59   40.333   4.197  6.292000e+05  143561.048345        4.82148
    16  15:29:59   40.654   5.263  2.566588e+05   60443.262720       75.23600
    17  15:29:59   47.172   5.285  4.432320e+05  100239.149376        4.68265
    18  15:29:59   58.230  10.575  6.556296e+05  137047.184608       46.20547
    19  15:29:59   32.014   4.833  4.169518e+05   95248.487576       12.00553
    20  15:29:59   54.555   5.651  4.800800e+05  108933.026479        6.85216
    21  15:29:59   93.472  10.013  2.401720e+06  551024.743092        5.39134
    22  15:29:59   75.828   5.588  7.299050e+05  148434.820650       14.44802
    23  15:29:59   43.758   5.675  1.295953e+06  125443.124680        2.57513
    24  15:29:59   44.956   4.227  3.272800e+06  292763.619350        6.72033
    25  15:29:59   51.287  18.541  3.508000e+05   79778.059000       19.34394
    26  15:29:59   75.840   5.607  6.294088e+05  142758.064848        3.61567
    27  15:29:59   45.689   4.686  2.926960e+05   66542.905884        7.87381
    28  15:29:59   86.061   7.041  1.712102e+06  160272.166485        4.49133
    29  15:29:59   67.654   7.180  7.206623e+05   79421.824469       19.17163
    30  15:29:59   70.789   7.979  2.211826e+06  200811.028600        4.85202
    31  15:29:59   69.640   8.174  7.138100e+05   78640.992633       22.04926
    32  15:29:59   70.800   4.215  6.566222e+05  149461.023120        7.80536
    33  15:29:59   90.939   5.405  1.363972e+06  122719.781481        4.50857
    34  15:29:59   37.810   6.709  8.870685e+05   99732.922857       47.98871
    35  15:29:59   47.625   5.090  8.456304e+05  194102.345094       12.40198
    36  15:29:59   56.056   4.843  4.590334e+05  104090.913408        6.80873
    37  15:29:59   70.978   6.308  1.320200e+06  302449.489538        7.61816
    38  15:29:59   30.023   5.872  7.379898e+05  170954.178755       34.16958
    39  15:29:59   52.268   7.828  4.115560e+05   83779.339736       47.22149
    40  15:29:59   40.597   8.518  3.126000e+05   63633.081072       75.90533
    41  15:29:59   43.561   5.363  5.046675e+05  115465.435800       74.00409
    42  15:29:59  622.670  15.324  2.218510e+06  219569.316575        9.90976
    43  15:29:59   57.989   4.316  4.407200e+05   98938.543942        8.85955
    44  15:29:59    0.000  17.123  4.084800e+06  388614.570870       74.06878
    45  15:29:59   34.691   3.790  1.597940e+06  131740.765500       19.11187
    46  15:29:59   36.534   9.239  3.960880e+05   90504.179000        8.22486
    47  15:29:59   50.794  10.579  4.927333e+05  113015.558400       15.08424
    48  15:29:59   43.863   4.195  1.032939e+06  236401.028322        8.41284
    49  15:29:59   86.190  10.169  7.641820e+05   62900.767050       44.14852
    """
    big_df = pd.DataFrame()
    page_count = get_zh_kcb_page_count()
    zh_sina_stock_payload_copy = zh_sina_kcb_stock_payload.copy()
    for page in range(1, page_count+1):
        print(page)
        zh_sina_stock_payload_copy.update({"page": page})
        res = requests.get(
            zh_sina_kcb_stock_url,
            params=zh_sina_kcb_stock_payload)
        data_json = demjson.decode(res.text)
        big_df = big_df.append(pd.DataFrame(data_json), ignore_index=True)
    return big_df


def stock_zh_kcb_daily(symbol="sh688008", factor=""):
    """
    从新浪财经-A股获取某个股票的历史行情数据, 大量抓取容易封IP
    :param symbol: str e.g., sh600000
    :param factor: str 默认为空, 不复权; qfq, 前复权因子; hfq, 后复权因子;
    :return: pandas.DataFrame
    不复权数据
                日期     开盘价     最高价     最低价     收盘价        成交    盘后量      盘后额
    0   2019-07-22  91.300  97.200  66.300  74.920  58330685  40778  3055088
    1   2019-07-23  70.020  78.880  70.000  74.130  23906020  43909  3254974
    2   2019-07-24  74.130  76.550  72.500  75.880  21608530  23149  1756546
    3   2019-07-25  75.000  79.980  74.600  78.000  24626920  66921  5219838
    4   2019-07-26  76.780  76.780  70.300  71.680  16831530  49106  3519918
    ..         ...     ...     ...     ...     ...       ...    ...      ...
    67  2019-10-31  59.790  60.500  57.800  58.290   2886407   3846   224183
    68  2019-11-01  57.900  59.960  57.600  59.250   2246059      0        0
    69  2019-11-04  60.040  61.880  60.040  61.740   3945106   1782   110021
    70  2019-11-05  61.100  62.780  60.850  62.160   4187105    400    24864
    71  2019-11-06  62.320  62.620  60.900  61.130   2331354   1300    79469

    后复权因子
             date          hfq_factor
    0  2019-07-22  1.0000000000000000
    1  1900-01-01  1.0000000000000000

    前复权因子
                 date          qfq_factor
    0  2019-07-22  1.0000000000000000
    1  1900-01-01  1.0000000000000000
    """
    res = requests.get(zh_sina_kcb_stock_hist_url.format(symbol, datetime.datetime.now().strftime("%Y_%m_%d"), symbol))
    data_json = demjson.decode(res.text[res.text.find("["):res.text.rfind("]")+1])
    data_df = pd.DataFrame(data_json)
    data_df.columns = ["日期", "开盘价", "最高价", "最低价", "收盘价", "成交", "盘后量", "盘后额"]
    if not factor:
        return data_df
    if factor == "hfq":
        res = requests.get(zh_sina_kcb_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])['data'])
        hfq_factor_df.columns = ["date", "hfq_factor"]
        return hfq_factor_df
    if factor == "qfq":
        res = requests.get(zh_sina_kcb_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])['data'])
        qfq_factor_df.columns = ["date", "qfq_factor"]
        return qfq_factor_df


if __name__ == "__main__":
    stock_zh_kcb_daily_df = stock_zh_kcb_daily(symbol="sh688008", factor="qfq")
    print(stock_zh_kcb_daily_df)
    stock_zh_kcb_spot_df = stock_zh_kcb_spot()
    print(stock_zh_kcb_spot_df)

==> akshare-0.4.84.tar.gz: akshare-0.4.84/akshare/stock/zh_stock_kcb_sina.py <==
# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Author: Albert King
date: 2019/10/30 11:28
contact: jindaxiang@163.com
desc: 新浪财经-科创板-实时行情数据和历史行情数据(包含前复权和后复权因子)
优化: 在科创板行情的获取上采用多线程模式(新浪会封IP, 不再优化)
"""
import datetime
import re

import demjson
import pandas as pd
import requests
from tqdm import tqdm

from akshare.stock.cons import (zh_sina_kcb_stock_payload,
                                zh_sina_kcb_stock_url,
                                zh_sina_kcb_stock_count_url,
                                zh_sina_kcb_stock_hist_url,
                                zh_sina_kcb_stock_hfq_url,
                                zh_sina_kcb_stock_qfq_url,
                                zh_sina_kcb_stock_amount_url)


def get_zh_kcb_page_count():
    """
    所有股票的总页数
    http://vip.stock.finance.sina.com.cn/mkt/#hs_a
    :return: int 需要抓取的股票总页数
    """
    res = requests.get(zh_sina_kcb_stock_count_url)
    page_count = int(re.findall(re.compile(r"\d+"), res.text)[0]) / 80
    if isinstance(page_count, int):
        return page_count
    else:
        return int(page_count) + 1


def stock_zh_kcb_spot():
    """
    从新浪财经-A股获取所有A股的实时行情数据, 大量抓取容易封IP
    http://vip.stock.finance.sina.com.cn/mkt/#qbgg_hk
    :return: pandas.DataFrame
    """
    big_df = pd.DataFrame()
    page_count = get_zh_kcb_page_count()
    zh_sina_stock_payload_copy = zh_sina_kcb_stock_payload.copy()
    for page in tqdm(range(1, page_count+1)):
        zh_sina_stock_payload_copy.update({"page": page})
        res = requests.get(
            zh_sina_kcb_stock_url,
            params=zh_sina_kcb_stock_payload)
        data_json = demjson.decode(res.text)
        big_df = big_df.append(pd.DataFrame(data_json), ignore_index=True)
    return big_df


def stock_zh_kcb_daily(symbol="sh688399", factor=""):
    """
    从新浪财经-A股获取某个股票的历史行情数据, 大量抓取容易封IP
    :param symbol: str e.g., sh600000
    :param factor: str 默认为空, 不复权; qfq, 前复权因子; hfq, 后复权因子;
    :return: pandas.DataFrame
    不复权数据
                日期     开盘价     最高价     最低价     收盘价        成交    盘后量      盘后额
    0   2019-07-22  91.300  97.200  66.300  74.920  58330685  40778  3055088
    1   2019-07-23  70.020  78.880  70.000  74.130  23906020  43909  3254974
    2   2019-07-24  74.130  76.550  72.500  75.880  21608530  23149  1756546
    3   2019-07-25  75.000  79.980  74.600  78.000  24626920  66921  5219838
    4   2019-07-26  76.780  76.780  70.300  71.680  16831530  49106  3519918
    ..         ...     ...     ...     ...     ...       ...    ...      ...
    67  2019-10-31  59.790  60.500  57.800  58.290   2886407   3846   224183
    68  2019-11-01  57.900  59.960  57.600  59.250   2246059      0        0
    69  2019-11-04  60.040  61.880  60.040  61.740   3945106   1782   110021
    70  2019-11-05  61.100  62.780  60.850  62.160   4187105    400    24864
    71  2019-11-06  62.320  62.620  60.900  61.130   2331354   1300    79469

    后复权因子
             date          hfq_factor
    0  2019-07-22  1.0000000000000000
    1  1900-01-01  1.0000000000000000

    前复权因子
                 date          qfq_factor
    0  2019-07-22  1.0000000000000000
    1  1900-01-01  1.0000000000000000
    """
    res = requests.get(zh_sina_kcb_stock_hist_url.format(symbol, datetime.datetime.now().strftime("%Y_%m_%d"), symbol))
    data_json = demjson.decode(res.text[res.text.find("["):res.text.rfind("]")+1])
    data_df = pd.DataFrame(data_json)
    data_df.index = pd.to_datetime(data_df["d"])
    data_df.index.name = "date"
    del data_df["d"]

    r = requests.get(zh_sina_kcb_stock_amount_url.format(symbol, symbol))
    amount_data_json = demjson.decode(r.text[r.text.find("["): r.text.rfind("]")+1])
    amount_data_df = pd.DataFrame(amount_data_json)
    amount_data_df.index = pd.to_datetime(amount_data_df.date)
    del amount_data_df["date"]
    temp_df = pd.merge(data_df, amount_data_df, left_index=True, right_index=True, how="left")
    temp_df.fillna(method="ffill", inplace=True)
    temp_df = temp_df.astype(float)
    temp_df["amount"] = temp_df["amount"] * 10000
    temp_df["turnover"] = temp_df["v"] / temp_df["amount"]
    temp_df.columns = ["开盘价", "最高价", "最低价", "收盘价", "成交", "盘后量", "盘后额", "流通股本", "换手率"]
    temp_df.columns = ["open", "high", "low", "close", "volume", "after_volume", "after_amount", "outstanding_share", "turnover"]
    if not factor:
        return temp_df
    if factor == "hfq":
        res = requests.get(zh_sina_kcb_stock_hfq_url.format(symbol))
        hfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])['data'])
        hfq_factor_df.columns = ["date", "hfq_factor"]
        hfq_factor_df.index = pd.to_datetime(hfq_factor_df.date)
        del hfq_factor_df["date"]
        return hfq_factor_df
    if factor == "qfq":
        res = requests.get(zh_sina_kcb_stock_qfq_url.format(symbol))
        qfq_factor_df = pd.DataFrame(
            eval(res.text.split("=")[1].split("\n")[0])['data'])
        qfq_factor_df.columns = ["date", "qfq_factor"]
        qfq_factor_df.index = pd.to_datetime(qfq_factor_df.date)
        del qfq_factor_df["date"]
        return qfq_factor_df


if __name__ == "__main__":
    stock_zh_kcb_daily_qfq_df = stock_zh_kcb_daily(symbol="sh688399", factor="qfq")
    print(stock_zh_kcb_daily_qfq_df)
    stock_zh_kcb_daily_hfq_df = stock_zh_kcb_daily(symbol="sh688399", factor="hfq")
    print(stock_zh_kcb_daily_hfq_df)
    stock_zh_kcb_daily_df = stock_zh_kcb_daily(symbol="sh688399", factor="")
    print(stock_zh_kcb_daily_df)
    # stock_zh_kcb_spot_df = stock_zh_kcb_spot()
    # print(stock_zh_kcb_spot_df)
